


;***************************************************************************
;*
;* "div48s" - 48/48 Bit Unsigned Division
;*
;*Divided	wird in DD_5....0 abgelegt( r16..r21) 
;*Divisor	wird in DIV_5...0 abgelegt (r22..r27)
;*
;*Ergebnis	wird in DD_5...0 abgelegt, bzw. DRES_0...5 (r16..r21)
;*Rest		wird in DREM_0....5abgelegt (r28..r31 und r14,15 )
;*
;*
;*Vorher mssen natrlich diese Register gesichert werden 
;*dh.: DD_3..0, DIV_3...0 und DREM_3...0
;*dh alle Register von r14..r31 werden verndert und mssen vorher gesichert werden
;*
;*
;*Register Counter r13 wird von Funktion selbst gesichert
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	DD_div48_5	=r16		;Dividend MSB
.def	DD_div48_4	=r17		;
.def	DD_div48_3	=r18		;
.def	DD_div48_2	=r19		;
.def	DD_div48_1	=r20		;
.def	DD_div48_0	=r21		;LSB



.def	DIV_div48_5	=r22		;Divisor MSB
.def	DIV_div48_4	=r23		;
.def	DIV_div48_3	=r24		;
.def	DIV_div48_2	=r25		;
.def	DIV_div48_1	=r26		;
.def	DIV_div48_0	=r27		;LSB



.def	DREM_div48_5	=r28		;Rest der Division MSB
.def	DREM_div48_4	=r29		;
.def	DREM_div48_3	=r30		;
.def	DREM_div48_2	=r31		;
.def	DREM_div48_1	=r14		;
.def	DREM_div48_0	=r15		;LSB


.def	DRES_div48_5	=r16		;result byte 3	
.def	DRES_div48_4	=r17		;result byte 2
.def	DRES_div48_3	=r18		;result byte 1
.def	DRES_div48_2	=r19		;result byte 0
.def	DRES_div48_1	=r20		;result byte 1
.def	DRES_div48_0	=r21		;result byte 0



.def	COUNTER_div48	=r13		;loop COUNTER_div48


;****************************************************** 


;****************************************************** Code

div48s:	

	;Sichern der Register fr Remainder und des COUNTER_div48s
	Push	COUNTER_div48
	

	;Laden des COUNTER_div48s (mit N+1=49 )
	ldi	r28,49
	mov	COUNTER_div48,r28


	;Lschen des Remainders/Restregister
	clr	DREM_div48_5
	clr	DREM_div48_4
	clr	DREM_div48_3
	clr	DREM_div48_2
	clr	DREM_div48_1
	clr	DREM_div48_0

	;Lschen des Carry-Bits
	clc


	
div48_mmarke1:
	;Rotate Divident mit Carry
	rol	DD_div48_0
	rol	DD_div48_1
	rol	DD_div48_2
	rol	DD_div48_3
	rol	DD_div48_4
	rol	DD_div48_5


	;Dekrementiere COUNTER_div48
	dec	COUNTER_div48

	;Wenn COUNTER_div48 = Null dann beende
	;sonst springe zu div48_marke2
	brne	div48_mmarke2

	;sonst

	;Alte Werte im COUNTER_div48 Register zurck
	pop 	COUNTER_div48

	;Rcksprung
	ret


div48_mmarke2:
	;Rotate Remainder nach links mit Carry
	rol	DREM_div48_0
	rol	DREM_div48_1
	rol	DREM_div48_2
	rol	DREM_div48_3
	rol	DREM_div48_4
	rol	DREM_div48_5


	;Subtrahiere Reminder-Divisor
	sub	DREM_div48_0,DIV_div48_0
	sbc	DREM_div48_1,DIV_div48_1
	sbc	DREM_div48_2,DIV_div48_2
	sbc	DREM_div48_3,DIV_div48_3
	sbc	DREM_div48_4,DIV_div48_4
	sbc	DREM_div48_5,DIV_div48_5



	
	;Wenn Carrybit nicht gesetzt ist (also noch positiv) dann Springe zu div48_Marke3 und 	
	brcc	div48_mmarke3

	;wenn Carry nicht gesetzt (also schon negatives Ergebnis) dann:
	;Rckaddition
	add	DREM_div48_0,DIV_div48_0
	adc	DREM_div48_1,DIV_div48_1
	adc	DREM_div48_2,DIV_div48_2
	adc	DREM_div48_3,DIV_div48_3
	adc	DREM_div48_4,DIV_div48_4
	adc	DREM_div48_5,DIV_div48_5


	
	;Lsche Carrybit und springe auch zu div48_marke1
	clc
	rjmp	div48_mmarke1	

	

	
	
div48_mmarke3:	
	;setze Carry-Bit
	sec
	;und springe wieder zu div48_marke1	
	rjmp	div48_mmarke1
	








;***************************************************************************
;***************************************************************************


;***************************************************************************
;*
;* "div_fixpoint_48" -liefert schon reelles Ergebnis (Ganzer Anteil + Nachkommastellen)
;* 32/32 Bit Division 
;*
;*Divided	wird in DD_3.....0 abgelegt 	r16..r19
;*Divisor	wird in DIV_3....0 abgelegt	r20..r23
;*
;*Ergebnis	wird in DD_3....0 abgelegt, bzw. DRES_div48_0...3 r16..r19
;*
;*
;*
;*Vorher mssen natrlich diese Register gesichert werden 
;*dh.: DD_3..0, DIV_3...0 also r16..r23
;*Register COUNTER wird in Routine gesichert, man mu sich alter nicht vorher drum kmmern.
;*
;*
;*Diese Funktion bedient sich der vorherigen Divroutine 
;*Es wird hier mit Rest den die vorherige Fkt. unbercksichtigt lt einbezogen 
;*verarbeitet.
;*
;*diese Fkt. funktioniert nur, wenn der Divisor grer 1 ist
;***************************************************************************

div_fix_48:
	
	
	;Sichern der Register r24..r27 u. evt. r28..r31
	push	r13
	push	r14
	push	r15
	push	r20	;sichern Divisor
	push	r21
	push	r22
	push	r23
	push	r24
	push	r25
	push	r26
	push	r27
	push	r28
	push	r29
	push	r30
	push	r31	;15mal Push
	


	;Lschen der Register r24..r31
	;brauchen nicht mehr gelscht zu werden, das bernimmt jetzt die Div48s routine


	
	;Aufruf der ersten Division , Ergbnis gibt den ganzen Teil des Ergebnisses wieder
		rcall	div32s	;Divisor ist immer noch unverndert in r20..r23 (nach Division)

	;jetzt die ganzen in r19 u. r18 - diese sichern auf Stack
		push	r19
		push 	r18	;17 Push
		

	
	;Rest der 1. Division ist jetzt in reg24..r27 gespeichert
	mov	r16,r24		;ganzen Rest um 16 Bit nach links verschieben dh. die ganzen vorderen 32 Bit ausnutzen
	mov	r17,r25		;die 16 Nachkommastellenbits bleiben Null
	mov	r18,r26
	mov	r19,r27

	
	mov	r27,r23		;Divisor laden
	mov	r26,r22
	mov	r25,r21
	mov	r24,r20
	
	clr	r20		;Kommastellen des Dividenten gleich Null
	clr	r21
	
	clr	r22		;die oberen 2 Byte des Divisor sind Null
	clr	r23

	rcall	div48s		;Division aufrufen
	
	;jetzt die Werte in Register 20 u. 21 dh. Nachk
	;mssen nach reg 18 u. 19 verschoben werden
	mov	r19,r21
	mov	r18,r20
	
	;jetzt noch den Ganzen Anteil vom Stack holen und in r16 u. r17 speichern
	pop	r16
	pop	r17




	;Jetzt zurckschreiben der Register 
	pop	r31
	pop	r30
	pop	r29
	pop	r28
	pop	r27
	pop	r26
	pop	r25
	pop	r24
	pop	r23
	pop	r22
	pop	r21
	pop	r20
	pop	r15
	pop	r14
	pop	r13

	ret				;SCHEISSE      hier stand RETI

;Um diese Divsion bei Bedarf noch genauer zu machen mu entweder :
;	-Zahlenbereich von 32Bit erweitert werden (zB. 48 oder 64 Bit)
;	-oder die Bytes des Restes einzeln bearbeiten(dividieren)  und diese also maximal nach links schieben
;	 dh. ganz linkes Byte des Restes um 40 statt um 32
;	     das zweit kleinste um 32
;	     das kleinste Vorkomma um 24	
;	     das grte Vorkommabyte um 16 Bit	
; 	ich denke aber, das die Ergebnisse fr unseren Anwenderfall ausreichend sind
	
;***************************************************************************
		




;***************************************************************************
;*
;* "div32s" - 32x32 Bit Unsigned Division
;*
;*Divided	wird in DD_3.....0 abgelegt (r16..r19)
;*Divisor	wird in DIV_3....0 abgelegt (r20..r23)
;*
;*Ergebnis	wird in DD_3....0 abgelegt, DRES3..0 (r16..r19) abgelegt
;*Rest		wird in DREM_0....3 abgelegt (r24..r27)
;*
;*
;*Vorher mssen natrlich diese Register gesichert werden 
;*dh.: DD_3..0, DIV_3...0 und DREM_3...0 (also alle Register r16..r27)
;*Register COUNTER wird in Routine gesichert, man mu sich alter nicht vorher drum kmmern.
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	DD_div32_3	=r16		;Dividend
.def	DD_div32_2	=r17		;
.def	DD_div32_1	=r18		;
.def	DD_div32_0	=r19		;

.def	DIV_div32_3	=r20		;Divisor
.def	DIV_div32_2	=r21		;
.def	DIV_div32_1	=r22		;
.def	DIV_div32_0	=r23		;

.def	DREM_div32_3	=r24		;Rest
.def	DREM_div32_2	=r25		;
.def	DREM_div32_1	=r26		;
.def	DREM_div32_0	=r27		;

.def	DRES_div32_3	=r16		;result byte 3
.def	DRES_div32_2	=r17		;result byte 2
.def	DRES_div32_1	=r18		;result byte 1
.def	DRES_div32_0	=r19		;result byte 0

.def	COUNTER_div32	=r28		;loop COUNTER_div32


;****************************************************** 


;****************************************************** Code

div32s:	

	;Sichern der Register fr Remainder und des COUNTER_div32s
	Push	COUNTER_div32

	;Lschen des Remainders
	clr	DREM_div32_3
	clr	DREM_div32_2
	clr	DREM_div32_1
	clr	DREM_div32_0

	;Lschen des Carry-Bits
	clc


	;Laden des COUNTER_div32s (mit N+1=17 )
	ldi	COUNTER_div32,33

	
	
mmarke1:
	;Rotate Divident mit Carry
	rol	DD_div32_0
	rol	DD_div32_1
	rol	DD_div32_2
	rol	DD_div32_3

	;Dekrementiere COUNTER_div32
	dec	COUNTER_div32

	;Wenn COUNTER_div32 = Null dann beende
	;sonst springe zu marke2
	brne	mmarke2

	;sonst

	;Alte Werte im COUNTER_div32 Register zurck
	pop 	COUNTER_div32

	;Rcksprung
	ret


mmarke2:
	;Rotate Remainder nach links mit Carry
	rol	DREM_div32_0
	rol	DREM_div32_1
	rol	DREM_div32_2
	rol	DREM_div32_3

	;Subtrahiere Reminder-Divisor
	sub	DREM_div32_0,DIV_div32_0
	sbc	DREM_div32_1,DIV_div32_1
	sbc	DREM_div32_2,DIV_div32_2
	sbc	DREM_div32_3,DIV_div32_3

	
	;Wenn Carrybit nicht gesetzt ist (also noch positiv) dann Springe zu Marke3 und 	
	brcc	mmarke3

	;wenn Carry nicht gesetzt (also schon negatives Ergebnis) dann:
	;Rckaddition
	add	DREM_div32_0,DIV_div32_0
	adc	DREM_div32_1,DIV_div32_1
	adc	DREM_div32_2,DIV_div32_2
	adc	DREM_div32_3,DIV_div32_3
	
	;Lsche Carrybit und springe auch zu marke1
	clc
	rjmp	mmarke1	

	

	
	
mmarke3:	
	;setze Carry-Bit
	sec
	;und springe wieder zu marke1	
	rjmp	mmarke1
	




;***************************************************************************
;*
;* "wurzel32s" 
;*
;*Wurzelfunktion hat zur Zeit eine Genauigkeit von 3 Stellen nach dem Komma
;*hhere Genauigkeit durch verbesserte Divisionsroutine (zBsp. auf 48 Bit erhhen)--> dadurch kann dann der Schwellwert noch weiter vermindert werden
;*Ausweitung des Zahlenbereichs von :16.16 Bit (also 32Bit) auf vielleicht 16.32 Bit (also 48 Bit)
;*vielleicht ist die Routine aus so ausreichend
;*
;*32Bit Zahl X aus der die Wurzel gezogen werden soll (Ausgangszahl wird als X bezeichnet und Ergebnis als Y )
;*
;*
;*X	wird in X_3....0 abgelegt  (reg16...reg19) Eingangswert
;*Y	wird in Y_3....0 abgelegt (reg20...reg23)  Ergebnis
;*
;*
;*
;*Vorher mssen natrlich diese Register gesichert werden 
;*dh.: IN3_3..0, OUT_3..0
;*
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	X_3	=r16		;Eingangswert
.def	X_2	=r17		;
.def	X_1	=r18		;
.def	X_0	=r19		;

.def	Y_3	=r20		;Ergebnis
.def	Y_2	=r21		;
.def	Y_1	=r22		;
.def	Y_0	=r23		;

.def	Schr_3	=r24		;
.def	Schr_2	=r25		;
.def	Schr_1	=r26		;
.def	Schr_0	=r27		;

.def	Sicher_3=r28		;Registern fr Zwichensicherung
.def	Sicher_2=r29
.def	Sicher_1=r30
.def	Sicher_0=r31

.def	Loop_Counter=r15	;Zhler der Iterationsdurchlufe
;****************************************************** 
;Schwell-Werte zum Abbrechen der Funktion !!!!
;Hier wird Genauigkeit festgelegt.

.equ	Schranke_03=0x00
.equ	Schranke_02=0x00
.equ	Schranke_01=0x01	
.equ	Schranke_00=0x00

;Anfangswert der Iteration fr Y
.equ	Start_03=0x00	;Startwert 1.1
.equ	Start_02=0x01
.equ	Start_01=0x19	
.equ	Start_00=0x99

.equ	max_loops=50	;Wenn nach 50 Durchlufen der Schwellwert noch nicht erreicht ist wird abgebrochen. Wahrscheinlich ist dann nur ein Fehler im Programm
			;oder der Schwellwert zu niedrig angesetzt.
;****************************************************** Code

wurzel32s:	
	;Lade Iterationszhler
	push	Loop_Counter	;Sicher Register r15
	push	Schr_3
	push	Schr_2
	push	Schr_1
	push	Schr_0
	push	Sicher_3
	push	Sicher_2
	push	Sicher_1
	push	Sicher_0



	ldi	Y_3,max_loops
	mov	Loop_Counter,Y_3;Lade LoopCounter
	
;Lade Startwert in Y = 1.1 (dec) 
	;Y wird deshalb nicht gesichert, weil es doch Ergebnisregister ist, und also schon frei sein mu !!
	ldi	Y_3,Start_03
	ldi	Y_2,Start_02
	ldi	Y_1,Start_01
	ldi	Y_0,Start_00



wurzel_1:

	;Division Z=X/Y  x(r16...19) y(r20..r23) z(16...19)
	;sichern von x
	push	X_3
	push	X_2
	push	X_1
	push	X_0

	rcall	div_fix_48


	;Addition Y=Y+Z
	;Z in r16...r19 Y noch in r20..r23
	add	X_0,Y_0
	adc	X_1,Y_1
	adc	X_2,Y_2
	adc	X_3,Y_3
			;jetzt neuer Wert Y in X_0...3 also r16...19 gespeichert
	

	;Division Y=Y/2
	clr	Y_0	;lade in Divisor die 2
	clr	Y_1
	ldi	Y_2,2
	clr	Y_3	

	rcall div_fix_48;Division Y/2 Ergebnis ist neues Y und in r16...19 (also X_3..0)
	
	
	;ZS=Y*Y
	mov	Y_3,X_3		;lade jetzt Y als zweiten Faktor
	mov	Y_2,X_2
	mov	Y_1,X_1
	mov	Y_0,X_0		;jetzt Y in r16..19 und r20...r23
	
	
		;Y jetzt noch sichern in die Register r28...31
		mov	Sicher_3,Y_3
		mov	Sicher_2,Y_2
		mov	Sicher_1,Y_1
		mov	Sicher_0,Y_0

	;aufrufen der Multiplikationsroutine
		rcall	mpy32s 	;jetzt das Ergebnis in r16...r23
	
	;wenn Register 17 u. 16 belegt, dann Zahl schon viel zu gro
		cpi	X_3,1	
		brsh	weiter44
		cpi	X_2,1
		brsh	weiter44
		



	;Subtraktion von X-Multiplikationsergebnis (erstmal nur reg18...21)
		;reg18...r21 nach r23...r27
		mov	r24,r18		;die Vorkommazahlen	wenn Ergebnis grer als 32 Bit wird es vorher abgefangen
		mov	r25,r19		;die Vorkommazahlen
		mov	r26,r20		;die Nachkommazahlen
		mov	r27,r21		;die Nachkommazahlen reg 22. und 23 werden abgeschnitten

		;X von Stack und in reg16...19
		pop	X_0
		pop	X_1
		pop	X_2
		pop	X_3
		
		;nochmaliges sichern von x
		push	X_3
		push	X_2
		push	X_1
		push	X_0

	

		
	;Vor der Subtraktion die grere der beiden Zahlen finden !!!! Diese wird dann Minuend. Die kleinere Subtrahend.
	;jetzt endlich die Subtraktion und Ablegen in X_0...3

			cp	X_3,r24
				BREQ	next_vergleich1
				BRSH	X_gr_Schr
				RJMP	Schr_gr_X
			

	next_vergleich1:
			cp	X_2,r25
				BREQ	next_vergleich2
				BRSH	X_gr_Schr
				RJMP	Schr_gr_X

	next_vergleich2:
			cp	X_1,r26
				BREQ	next_vergleich3
				BRSH	X_gr_Schr
				RJMP	Schr_gr_X

	next_vergleich3:
			cp	X_0,r27
				BREQ	X_gr_Schr	;knnte aber noch extra Auswertung folgen weil Ergebnis= Null schon feststeht.
				BRSH	X_gr_Schr	;Wrde Zeit sparen aber nicht Speicher
				RJMP	Schr_gr_X

	

	X_gr_Schr:
			sub	X_0,r27
			sbc	X_1,r26
			sbc	X_2,r25
			sbc	X_3,r24
			rjmp	vergleich_schranke

	Schr_gr_X:
			sub	r27,X_0
			sbc	r26,X_1
			sbc	r25,X_2
			sbc	r24,X_3

			mov	X_0,r27
			mov	X_1,r26
			mov	X_2,r25
			mov	X_3,r24
			rjmp	vergleich_schranke


	X_gl_Schr:


vergleich_schranke:
	;Dekrementieren des Loop_Counters und Prfen auf Null
	dec	Loop_Counter
	BREQ	ende


	;jetzt bruchte ich die Schrankenwerte - Frage nur, wo sind die jetzt gerade ???
	;erst jetzt laden nach r24...r26
	;Lade Schrankenwert	
	ldi	Schr_3,Schranke_03
	ldi	Schr_2,Schranke_02
	ldi	Schr_1,Schranke_01
	ldi	Schr_0,Schranke_00

	
	;jetzt Schrankenwerte r24..r27
	;Y in r28..r31
	;X auf Stack
	;Subtraktionsergebnis in r16..r19

	;Vergleich mit Schranke
	
	;Vergleich Byte_3
		cp	Schr_3,X_3	;Vergleich von Subtraktionsergebnis u. Schranke
		BREQ	weiter1		;wenn Subtraktionsergebnis gleich dann weiter;also nchste Byte wird verglichen
		cp	Schr_3,X_3	
		BRLO	weiter4	;nochmaliger Vergleich, wenn schranke kleiner also X_3 dann nchster Duchlauf
		rjmp	ende		;wenn Schranke grer ist Beenden des Programms

weiter1:
	;Vergleich Byte_2
		cp 	Schr_2,X_2
		BREQ	weiter2		;wenn gleich gro
		cp 	Schr_2,X_2
		BRLO	weiter4		;wenn Schranke kleiner als Ergebnis
		rjmp	ende
weiter44:
		rjmp	weiter4

weiter2:
	;Vergleich Byte_1
		cp 	Schr_1,X_1
		BREQ	weiter3
		cp 	Schr_1,X_1
		BRLO	weiter4
		rjmp	ende

weiter3:
	;Vergleich Byte_0
		cp 	Schr_0,X_0
		BRLO	weiter4
		rjmp	ende


weiter4:
	;dh. das Subtraktionsergebnis war noch grer als Schwell, df. nchste Schleife beginnt
	;Verschieben von Y(jetzt r28...31) nach r20..r23
	mov	Y_3,r28
	mov	Y_2,r29
	mov	Y_1,r30
	mov	Y_0,r31

	
	;Holen von X aus dem Stack
	pop	X_0
	pop	X_1
	pop	X_2
	pop	X_3

	rjmp 	wurzel_1



ende:

	;Speichern des Ergebnisses in r20...23
	mov 	r20,r28
	mov 	r21,r29
	mov 	r22,r30
	mov 	r23,r31
	

	;Zurckholen des Stacks	
	pop	X_0
	pop	X_1
	pop	X_2
	pop	X_3

	pop 	Sicher_0
	pop 	Sicher_1
	pop 	Sicher_2
	pop 	Sicher_3
	pop	Schr_0
	pop	Schr_1
	pop	Schr_2
	pop	Schr_3
	pop 	Loop_Counter


	;Rcksprung aus Unterfunktion
	ret			





;***************************************************************************
;*
;* "mpy32s" - 32x32 Bit Unsigned Multiplication
;*
;*M1 wird in MCR0..3 abgelegt (MCR_0 ist low, MCR_3 High-Byte dh r20..r23)
;*M2 wird in AC_0...AC_3 abgelegt (AC_0 ist low, AC_3 High-Byte dh r16..r19)
;*
;*Ergebnis in r16..r23 AC_3...AC0 MQR_3...MQR_0
;*
;*
;*Vorher mssen natrlich diese Register gesichert werden 
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	AC_3	=r16		;multiplicand low byte
.def	AC_2	=r17		;multiplicand high byte
.def	AC_1	=r18		;multiplier low byte
.def	AC_0	=r19		;multiplier high byte
.def	MQR_3	=r20		;result byte 0 (LSB)
.def	MQR_2	=r21		;result byte 1
.def	MQR_1	=r22		;result byte 2
.def	MQR_0	=r23		;result byte 3 (MSB)
.def	SR_3	=r24		;multiplicand low byte
.def	SR_2	=r25		;multiplicand high byte
.def	SR_1	=r26		;multiplier low byte
.def	SR_0	=r27		;multiplier high byte
.def	M2_3	=r28		;result byte 0 (LSB)
.def	M2_2	=r29		;result byte 1
.def	M2_1	=r30		;result byte 2
.def	M2_0	=r31		;result byte 3 (MSB)

.def	Counter	=r15		;loop counter

;****************************************************** 


;****************************************************** Code

mpy32s:	

	;Sichern der Register M2_0..3 und SR_0..3
	Push	SR_3
	Push	SR_2
	Push	SR_1
	Push	SR_0
	Push	M2_3
	Push	M2_2
	Push	M2_1
	Push	M2_0


	;Sichere die Speicherzelle von Adresse StoreCount auf Stack
	
	push	Counter ; reg15 sichern

	

	;verschieben des Mulp2 von AC nach M2		
	mov	M2_0,AC_0	
	mov	M2_1,AC_1
	mov	M2_2,AC_2
	mov	M2_3,AC_3



	;Laden des Counters (also Speicherzelle im SRAM) 	
	ldi	AC_0,32
	mov 	Counter,AC_0;	schreiben des Zhler wertes in die Speicherzelle zum ERSTEN MAL  !!!!!!!


	;clear the AC Registers
	clr	AC_0		
	clr	AC_1
	clr	AC_2
	clr	AC_3


	



marke1:

	mov	SR_0,M2_0	;Kopieren der M2_0..3 zu SR_0...3
	mov	SR_1,M2_1
	mov	SR_2,M2_2
	mov	SR_3,M2_3

	;Zugriff auf ganz linkes Bit 
	SBRC	MQR_0,0		;wenn ganz rechtes Bit=1 dann Sprung zu marke2, dh. Addieren von AC+SR
	RJMP	marke2

marke3:
	;Schieben von AC und MQR nach rechts
	clc			;Lsche Carrybit vor dem Rotate

	ROR	AC_3
	ROR	AC_2
	ROR	AC_1
	ROR	AC_0
	ROR	MQR_3		;schieben nach rechts Versust von Bit 0	
	ROR	MQR_2		;schieben nach rechts Versust von Bit 0	
	ROR	MQR_1		;schieben nach rechts Versust von Bit 0	
	ROR	MQR_0		;schieben nach rechts Versust von Bit 0	

	


	;Test ob Zhler schon Null ist

	
	;Dekrementieren des Zhlers
	DEC	Counter		;Dekrementiere Zhlerstand

	BRNE	marke1		;wenn nicht Null dann Sprung nach marke1

	
	;Zhlerwert ist Null 

	;Rcksprung aus Routine
	;vorher noch Rckschreiben von Original-$60 nach SRAM Speicher
	POP	Counter 		;dh vom Stack holen

	;Rckschreiben der anderen Register
	Pop	M2_0
	Pop	M2_1
	Pop	M2_2
	Pop	M2_3
	Pop	SR_0
	Pop	SR_1
	Pop	SR_2
	Pop	SR_3

	ret			;Rcksprung aus Unterprogramm





marke2:
	;Addieren von AC=AC+SR
	ADD	AC_0,SR_0	;Addieren von ADC_0+SR_0 u. speichern in ADC_0
	ADC	AC_1,SR_1	
	ADC	AC_2,SR_2	
	ADC	AC_3,SR_3	
	
	RJMP	marke3		;Rcksprung zu marke3 und weiter mit rechtsschieben










;***************************************************************************
;***************************************************************************

;***************************************************************************

;***************************************************************************

