; This is the beginning

.include "2313def.inc"

.equ	l_cs	=4
.equ	sio	=8
.equ	sc	=16
.equ	fout	=8

.equ	bit_l_cs	=2
.equ	bit_sio		=3
.equ	bit_sc		=4
.equ	bit_fout	=3

.device	AT90S2313

.eseg
;here comes the default values for the dds system
.org 0
ee_dds_d:		.db	0x7f,0xcb,0x91,0x00	;increment for 1/2 dds count  = 12kHz
ee_dds_x:		.db	0x4e,0xa4,0xa8		;increment for 1 step dds_counter
ee_dds_n:		.db	0x01,0xa0		;increment for oc counter
ee_dds_count32:		.db	0,0,0,0			;dds counter
ee_dds_oldmsb:		.db	0			;old msb of dds_counter
ee_K:			.db	0x02,0xB7,0x00,0x00	;THE K factor for f=K*sqrt(T)  = 695 
ee_offset:		.db	0x00,0xda,0x26,0x66	;offset of transfer function = 218,15 K
ee_slope:		.db	0x10,0x00		;slope of transfer function = 0.0625 K/LSB
ee_fref:		.db	0x2e,0xe0,0x0,0x0	;calibration frequency = 12000 Hz
ee_type:		.db	0x01			;type identifyer; 01=low cost, 02=precicion
ee_switch:		.db	0			;defines whether frequency gen. via PC or ADC
							;0-> ADC ; <>0-> PC (RS232)

.equ	datasize = 15					;size of ee_K ... ee_switch 
.equ	data_add = 14					;address of ee_K in EEPROM

.dseg
.org 0x60
dds_d:			.byte	4		;increment for 1/2 dds count
dds_x:			.byte	3		;increment for 1 step dds_counter
dds_n:			.byte	2		;increment for oc counter
dds_count32:		.byte	4		;dds counter
dds_oldmsb:		.byte	1		;old msb of dds_counter
K:			.byte	4		;THE K factor for f=K*sqrt(T)
offset:			.byte	4		;offset of transfer function
slope:			.byte	2		;slope of transfer function
fref:			.byte	4		;calibration frequency
type:			.byte	1		;type identifyer
switch:			.byte	1

.equ		temp_add=0x80

.org temp_add
temp:			.byte	32		;temporary storage

.def	Time = r0

.cseg		;here starts the code segment

.org 0
	rjmp	init
.org 4
	rjmp	dds_int			;interupt handler for DDS
	rjmp	RTC			;timer overflow, gives 6ms timing pulses

dds_int:

	push	r29
	in 	r29,SREG
	push	r29			;save SREG

	lds	r6,dds_count32+0
	lds	r7,dds_count32+1
	lds	r8,dds_count32+2
	lds	r9,dds_count32+3	;load dds_count32 into r6,7,8,r9

	lds	r3,dds_n
	lds	r4,dds_n+1		;load n into r3,4

	lds	r29,dds_d+3
	add	r9,r29			;add low order d
	lds	r29,dds_d+2
	adc	r8,r29			;add middle order d + carry
	lds	r29,dds_d+1
	adc	r7,r29			;add high order d + carry
	lds	r29,dds_d+0
	adc	r6,r29			;add high order d + carry
	

dds_cmp:
	lds	r5,dds_oldmsb		;load old MSByte
	mov	r29,r6			;copy actual MSByte
	andi	r29,128			;mask copy of actual MSByte
	eor	r5,r29			;compare MSBs if equal or not
	brne	dds_counter_ok		;if not equal, OK
					;if equal, add one step to DDS/OC counter
	lds	r29,dds_x+2
	add	r9,r29			;add low order x
	lds	r29,dds_x+1
	adc	r8,r29			;add middle order x +carry
	lds	r29,dds_x+0
	adc	r7,r29			;add middle order x +carry
	ldi	r29,0
	adc	r6,r29			;add carry
	
	ldi	r29,1
	add	r4,r29
	ldi	r29,0
	adc	r3,r29			;increment OC counter step
	rjmp	dds_cmp			;compare again
	
dds_counter_ok:

	sts	dds_count32+0,r6
	sts	dds_count32+1,r7
	sts	dds_count32+2,r8
	sts	dds_count32+3,r9	;save DDS_counter

	ldi	r29,128
	and	r6,r29
	sts	dds_oldmsb,r6		;save actual MSB (masked)

	in	r7,ocr1al		;load oc event 
	in	r6,ocr1ah
	
	add	r7,r4			;add n
	adc	r6,r3

	out	ocr1ah,r6
	out	ocr1al,r7		;write OC into I/O registers

dds_int_out:
	pop	r29
	out	SREG,r29		;restore SREG
	pop	r29

	reti


RTC:
	in 	r1,SREG			;buffer for saving SREG, not available for further programming

	inc	r0			;r0 contains the timestamp
					;so this register is not available for other things
	
	out	SREG,r1			;restore SREG
	reti

init:		
	ldi	r16,$df
	out	spl,r16			;set stack pointer

	rcall	read_EE			;copy default values from eeprom

	ldi	r16,64
	out	TCCR1A,r16		;set Timer 1 to toggle PB3
	out	ubrr,r16		;set baud rate to 9k6
	ldi	r16,16+8
	out	ucr,r16			;enable UART RxD and TxD	

	ldi	r16,2+4+16
	out	ddrd,r16		;config PortD
	ldi	r16,8+1			;PB0 ist just for testing
	out	ddrb,r16		;config PortB

	ldi	r16,128+64
	out	timsk,r16		;enable TOIE1 and OCIE1
	bset	7			;global interrupt enable
	ldi	r16,1
	out	tccr1b,r16		;set input of timer 1 to clk					
					;now DDS is running

main:

	sbic	usr,rxc			;check rs232
	rcall	check_input

	ldi	r16,152			;main loop	
	cp	time,r16		;152 clocks equals 1 second
	brne	main
	clr	time

	lds	r16,switch
	cpi	r16,0
	brne	main			;if switch<>0, we are in direct frequency mode

	rcall	read_adc

;calculate temperature, -> new dds values

;multiply adcvalue with slope (slope * 2^16; 0,0625=4096)

	mov	r19,r18
	mov	r18,r17
	clr	r17
	clr	r16		;move adc value to r16..r19

	clr	r20
	clr	r21
	lds	r22,slope
	lds	r23,slope+1	;copy slope to r20..r23

	rcall	mpy32s

;add offset (includes conversion to absolut temperature too)

	lds	r16,offset+3
	add	r23,r16
	lds	r16,offset+2
	adc	r22,r16
	lds	r16,offset+1
	adc	r21,r16
	lds	r16,offset+0
	adc	r20,r16
	
;calculate sqrt(T)

	mov	r16,r20
	mov	r17,r21
	mov	r18,r22
	mov	r19,r23

	rcall	wurzel32s

;multiply with K (includes calibration of masterclock too)

	lds	r16,K
	lds	r17,K+1
	lds	r18,K+2
	lds	r19,K+3

	rcall	mpy32s

;store fout in temp
	
	sts	temp+16,r18
	sts	temp+17,r19
	sts	temp+18,r20
	
;calculate dds_x (fout*2^8*429.4967296*2^16)

	clr	r16
	mov	r17,r18
	mov	r18,r19
	mov	r19,r20		;copy fout*2^8

	ldi	r20,0x01
	ldi	r21,0xad
	ldi	r22,0x7f
	ldi	r23,0x29	;(2^32/10^7*2^16)

	rcall	mpy32s

	sts	temp+4,r18
	sts	temp+5,r19
	sts	temp+6,r20

;calculate dds_n

	ldi	r16,0x4c
	ldi	r17,0x4b
	ldi	r18,0x40
	ldi	r19,0x00	;5*10^6*2^8

	clr	r20
	lds	r21,temp+16
	lds	r22,temp+17
	lds	r23,temp+18	;fout*2^8

	rcall	div32s

	sts	temp+7,r18	
	sts	temp+8,r19		

;calculate dds_d

	clr	r20
	lds	r21,temp+4
	lds	r22,temp+5
	lds	r23,temp+6	;dds_X

	rcall	mpy32s

	sts	temp+0,r20
	sts	temp+1,r21
	sts	temp+2,r22
	sts	temp+3,r23	

;activate new dds values

	bclr	7			;disable interrupts
	ldi	r16,9
	clr	r31
	ldi	r30,temp_add		;source pointer
	clr	r29
	ldi	r28,0x60		;target pointer
main_copy:
	ld	r17,z+
	st	y+,r17			
	dec	r16
	brne	main_copy
	bset	7			;enable interrupts

	ldi	r16,'O'
	out	udr,r16			;after everything is done, we send a signal to
					;the PC to tell them we are ready to handle
					;incomming commands without significiant
					;delays until the next frequency change (1s)
	rjmp	main

;******************************************************************
check_input:				;check incomming data and branch to commands
	in	r16,udr
	cpi	r16,'P'
	breq	cmd_P
	cpi	r16,'R'
	breq	cmd_R
	cpi	r16,'D'
	breq	cmd_D
	cpi	r16,'N'
	breq	cmd_N
	cpi	r16,'A'
	breq	cmd_A
check_end:
	ret

;********************************************************************
cmd_P:
	rjmp	cmd_P_main		;we need this, because breq can only jump 63 words

;*********************************************************************
cmd_N:					;switch back to normal mode (frequncy sythesis controled by
	clr	r16			;adc value
	sts	switch,r16		
	rjmp	check_end

;*********************************************************************
cmd_A:					;read adc directly
	rcall	read_adc
;transmit adc value to pc
	out	udr,r17			;send high order adcvalue to terminal
cmd_A_waitrs232:
	sbis	usr,5			;test if transmit buffer empty
	rjmp	cmd_A_waitrs232
	out	udr,r18			;send low order byte
	rjmp	check_end

;*********************************************************************
cmd_D:					;switch to direct frequency synthesis and set frequency
	clr	time			;we need a timeout counter
	ldi	r30,temp_add	
	ldi	r16,9			;read 9 bytes from rs232
cmd_D_wait:
	ldi	r17,3			;about 18ms for 9 bytes is timeout
	cp	time,r17
	breq	check_end		;when timeout, leave
	sbis	usr,rxc
	rjmp	cmd_D_wait
	in	r17,udr
	st	z+,r17
	dec	r16
	brne	cmd_D_wait
	ldi	r16,1
	sts	switch,r16		;switch to direct frequency synthesis
;now copy  dds_d,x,n to ram
	bclr	7			;disable interrupts
	ldi	r16,9
	clr	r31
	ldi	r30,temp_add
	clr	r29
	ldi	r28,0x60		;copy new dds_d,x,n
cmd_D_copy:
	ld	r17,z+
	st	y+,r17
	dec	r16
	brne	cmd_D_copy
	bset	7			;enable interrupts
	rjmp	check_end

;*********************************************************************
cmd_R:
	ldi	r16,datasize		;byte counter
	ldi	r17,data_add		;eeprom address
cmd_R_EERead:
	sbic	EECR,EEWE		;if EEWE not clear
	rjmp	cmd_R_EERead		;wait more
	out	EEAR,r17		;output address
	sbi	EECR,EERE		;set EEPROM Read strobe
cmd_R_EEwait:
	sbic	EECR,EERE
	rjmp	cmd_R_EEwait
	in	r18,EEDR		;get data
cmd_R_waitrs232:
	sbis	usr,UDRE		;test if transmit buffer empty
	rjmp	cmd_R_waitrs232
	out	udr,r18			;send byte					
	inc	r17
	dec	r16
	brne	cmd_R_EERead
	rjmp	check_end

;*********************************************************************
cmd_P_main:				;programms the EEprom and copys new EEprom values to RAM
	clr	Time			;we need a timeout counter
	clr	r31
	ldi	r30,temp_add	
	ldi	r16,datasize		;read bytes from rs232
	subi	r16,1			;the last byte (ee_type) is NOT programmable
cmd_P_wait:
	ldi	r17,datasize/3		;about 6ms for 3 bytes is timeout
	cp	time,r17
	brsh	check_end		;when timeout, leave
	sbis	usr,rxc
	rjmp	cmd_P_wait
	in	r17,udr
	st	z+,r17
	dec	r16
	brne	cmd_P_wait

;now copy all to EEprom
	ldi	r16,datasize		;byte counter
	subi	r16,1
	ldi	r17,data_add		;eeprom address
	clr	r31
	ldi	r30,temp_add		;source index
cmd_P_EEwrite:
	sbic	EECR,EEWE		;if EEWE not clear
	rjmp	cmd_P_EEwrite		;wait more
	out	EEAR,r17		;output address
	ld	r18,z+
	out	EEDR,r18
	sbi	EECR,EEMWE		;set EEPROM Master write enable
	sbi	EECR,EEWE		;set EEPRMON write enabele
	inc	r17			;increase target pointer
	dec	r16			;decrease byte counter
	brne	cmd_P_EEwrite

;now copy all values to RAM	
	bclr	7			;disable interrupts
	ldi	r16,datasize
	subi	r16,1
	clr	r31
	ldi	r30,temp_add		;source pointer
	clr	r29
	ldi	r28,data_add+0x60	;target pointer in RAM
cmd_P_copy:
	ld	r17,z+
	st	y+,r17
	dec	r16
	brne	cmd_P_copy
	bset	7			;enable interrupts
	rjmp	check_end

;******************************************
read_adc:				;read ADC
	sbi	portd,bit_l_cs		;set l_cs
	cbi	portd,bit_sc		;clear sc
	cbi	portd,bit_l_cs		;set l_cs to low	
	clr	r17			;r17 is our high data register
	clr	r18			;r18 is our low data register
	ldi	r16,13
adcloop:
	sbi	portd,bit_sc		;set sc
	clc				;clear carry flag
	rol	r18			;shift data register
	rol	r17
	sbic	pind,bit_sio		;skip if incoming bit is 0
	sbr	r18,1			;set bit in data register
	cbi	portd,bit_sc		;clear sc to clock out next bit
	dec	r16
	brne	adcloop
	sbi	portd,bit_l_cs		;pull l_cs high
	sbrc	r17,4
	ori	r17,32+64+128		;if negative, extend to 16 bit

;add 880 to adc value (we move the code for -55 C to 0 C to avoid signed multiplications)

	ldi	r16,0x70		;880 = 0x370
	add	r18,r16
	ldi	r16,0x03
	adc	r17,r16
	ret

;******************************************
read_EE:				;read default values from EEprom and copy it to RAM

	ldi	r16,30			;byte counter
	clr	r17			;eeprom address
	ldi	r30,0x60			
	clr	r31			;target index
EERead:
	sbic	EECR,EEWE		;if EEWE not clear
	rjmp	EERead			;wait more
	out	EEAR,r17		;output address
	sbi	EECR,EERE		;set EEPROM Read strobe
EEwait:
	sbic	EECR,EERE
	rjmp	EEwait

	in	r18,EEDR		;get data
	st	Z+,r18			;save data
	inc	r17
	dec	r16
	brne	EERead
	ret

.include "divide5.asm"
